/* Copy edges between neighboring processors */

#include "comext.h"
#include <stdio.h>

/* N.B. The syntax of exchange is

   exchange(ibuf, ilen, isrc,  itype,
            obuf, olen, odest, otype);

   which is equivalent to

   exwrite(obuf, olen, odest, otype);
   exread (ibuf, ilen, isrc,  itype);

   i.e. olen bytes are taken from obuf and written to the node odest, 
        while ilen bytes are read in from isrc and put into ibuf.
*/


spin_edges()
{
   int type = EDGE;
   int len = jsiz_short;
   int vlen = sizeof(short);
   int voffset = jmax_short;
   int items = isiz;
   int d1,d2;

   /* Only need spin edges in the positive direction */

   /* left edge -> right edge  (right node passes to left node) */
   d1 = exchange(&spin[isiz+1][1],len,&right,&type,
                 &spin[1][1],len,&left,&type);

   /* bottom edge -> top edge  (top node passes to bottom node) */
   d2 = exvchange(&spin[1][jsiz+1],vlen,voffset,items,&top,&type,
                  &spin[1][1],vlen,voffset,items,&bottom,&type);

#ifdef DEBUG
fmulti(stdout);
if ((d1 != len) || (d2 != items))
  printf("spin exchange %d %d  %d %d\n", d1,d2, len,items);
fflush(stdout);
fsingl(stdout);
#endif

}


cluster_edges()
{
int type = EDGE;
int len,vlen,voffset,items;
int d1,d2;

   /* Only need cluster edges in the positive direction */

   /* left edge -> right edge */
   len = jsiz_int;
   d1 = exchange(&cluster[isiz+1][1],len,&right,&type,
                 &cluster[1][1],len,&left,&type);

   /* bottom edge -> top edge */
   vlen = sizeof(int);
   voffset = jmax_int;
   items = isiz;
   d2 = exvchange(&cluster[1][jsiz+1],vlen,voffset,items,&top,&type,
                  &cluster[1][1],vlen,voffset,items,&bottom,&type);
   
#ifdef DEBUG
fmulti(stdout);
if ((d1 != len) || (d2 != items))
  printf("cluster exchange %d %d  %d %d\n", d1,d2,len,items);
fflush(stdout);
fsingl(stdout);
#endif

}



short get_spin(i,j)
int i,j;
{
int type = EDGE;
int len = sizeof(short);
int vlen = sizeof(short);
int voffset = jmax_short;
int items = 1;

   /* Get the value of the spin at site (i,j). This may 
      require some communication if (i,j) is on another
      processor */

   if (i > isiz) {
      exchange(&spin[isiz+1][j],len,&right,&type,
               &spin[1][j],len,&left,&type);
   }
   else if (i < 1) {
      exchange(&spin[0][j],len,&left,&type,
               &spin[isiz][j],len,&right,&type);
   }

   if (j > jsiz) {
      exvchange(&spin[i][jsiz+1],vlen,voffset,items,&top,&type,
                &spin[i][1],vlen,voffset,items,&bottom,&type);
   }
   else if (j < 1) {
      exvchange(&spin[i][0],vlen,voffset,items,&bottom,&type,
                &spin[i][jsiz],vlen,voffset,items,&top,&type);
   }

   return spin[i][j];
}

